# Memos of JupyterHub with SLURM Integration
## Setup
- Install JupyterHub to `/opt/jupyterhub` following documentation, set
  up authentication, reverse proxy to localhost:8000
- Set up a remote kernel as default kernel
  - In a **network location accessible by all users**, create directory,
    build virtual environment and **activate**.
  - `pip install ipython ipykernel remote_ikernel`, plus common packages
    - `numpy matplotlib pandas`
  - `cd lib/pythonsth/site-packages/remote-ikernel` and **optionally**
    make the following changes to `kernel.py`:
     - ```patch
       # Python Compatibility
       -        re_passphrase = re.search(b'Enter passphrase .*:', text)
       -        re_password = re.search(b'.*@.* password:', text)
       +        re_passphrase = re.search('Enter passphrase .*:', str(text))
       +        re_password = re.search('.*@.* password:', str(text))

       # Do not contaminate bash history
       -         srun_cmd = ('{srun} {tasks} -J {job_name} {args} -v --pty bash -i'
       +         srun_cmd = ('{srun} {tasks} -J {job_name} {args} -v --pty bash'

       # SLURM Compatibility
       -        srun.expect('srun: Node (.*), .* tasks started')
       +        srun.expect('Nodes (.*) are ready for job')

       ```
     - This change may be helpful if you need to change somewhere else to use shell, for e.g. load modules:
       - ```patch
                  if self.connection is None:
         -            self.connection = pexpect.spawn(command, timeout=timeout,
         -                                            logfile=self.log)
         +            self.connection = pexpect.spawn('/bin/bash', ['-c', command], timeout=timeout, encoding='utf-8')
         +            self.connection.logfile = self.log
         ```
  - Add the followings to `manage.py` to enable debugger for users:
    - ```patch
           if language is not None:
               kernel_json['language'] = language
      +        if re.search('python', language, re.IGNORECASE) is not None:
      +            kernel_json['metadata'] = {'debugger': True}
      ```
  - Copy `setup_remote_ikernel` to `/opt` and modify paths to match
    with setup
  - Generate configuration with `create_kernel.sh` and try it in
    JupyterHub to make sure it is working
     - Use `squeue -u $USER` and `journalctl -eu jupyterhub` for
       diagnosing
     - Avoid tunneling, especially when there is load balance on a
       domain
     - Check everything that is not using absoulte path
       - This wasted me tons of time :(
  - After verifying it works, move
    `~/.local/share/jupyter/kernels/[longstring]`
    to `/opt/jupyterhub/share/jupyter/kernels/`
      - Ask another person to try it! (IMPORTANT)
  - After verifying the global one works,
    `cd /opt/jupyterhub/share/jupyter/kernels && cp python3{,_old}`,
    and `mv {[longstring],python3}/kernel.json`
  - It SHALL be working right now; Congrats if you got here in one shot!
## Integrate with Custom Virtual Environments
After logging in, create a terminal on the website and cd to where the
virtual environment resides in, activate it, and run the following:
```bash
pip install ipython ipykernel
/opt/setup_remote_ikernel/create_kernel.sh 120 8 0
# Three numbers are time in minutes, number of CPUs, number of GPUs
# respectively
# Additional SLURM command may be appended following it, like -C A100
# or -p long, as in
# /opt/setup_remote_ikernel/create_kernel.sh 120 8 0 -C A100 -p long
```
Now refresh the page and choose the new kernel which is NOT marked as
GLOBAL. Packages installed in the regular way through command line will
be usable in the notebook. In addition, one may use `%pip install sth`
to perform installation in notebook. Kernels added can be managed in
`~/.local/share/jupyter/kernels/[long string]/kernel.json`. To remove a
kernel, delete corresponding directory in `kernels/` just mentioned.

Hope this would work :)
