#!/bin/bash
set -eu

CLUSTER="CLUSTERNAMEHERE"
SRUN_PATH=$(which srun)
REMOTE_IKERNEL_BIN='/SOME/VENV/PATH/bin/remote_ikernel'

NARG_MANDATORY=3
if [[ $# -lt $NARG_MANDATORY ]]; then
  echo "Usage: $0 JOB_TIME ALLOC_CPU ALLOC_GPU [Extra options for srun...]"
  exit 1
fi
JOB_TIME=$1
ALLOC_CPU=$2
ALLOC_GPU=$3
shift $NARG_MANDATORY
EXTRA=" $@"
if ! test "$JOB_TIME" -gt 1 2> /dev/null ; then
  echo JOB_TIME is not an integer greater than 1
  exit 1
fi
if ! test "$ALLOC_GPU" -ge 0 2> /dev/null ; then
  echo ALLOC_GPU is not a nonnegative integer
  exit 1
fi
if ! test "$ALLOC_CPU" -gt 0 2> /dev/null ; then
  echo ALLOC_CPU is not a positive integer
  exit 1
fi

GPU_STRING="NOGPU"
GPU_COMMAND=""
if test "$ALLOC_GPU" -gt 0 2> /dev/null ; then
  GPU_STRING="$ALLOC_GPU GPU"
  if test "$ALLOC_GPU" -gt 1 2> /dev/null ; then
    GPU_STRING="$GPU_STRING"'S'
  fi
  GPU_COMMAND="-G $ALLOC_GPU"
fi

VIRTUAL_ENV_PREFIX=""
set +u
if [[ ! -z $VIRTUAL_ENV ]]; then
  VIRTUAL_ENV_PREFIX="$VIRTUAL_ENV/bin/"
fi
set -u

"$REMOTE_IKERNEL_BIN" manage --add \
  --kernel_cmd="$VIRTUAL_ENV_PREFIX"'ipython kernel -f {connection_file}'  \
  --name="$CLUSTER $JOB_TIME Mins $GPU_STRING $EXTRA" --cpus="$ALLOC_CPU" \
  --interface=slurm --language=python \
  --launch-cmd="$SRUN_PATH" \
  --remote-launch-args="--time $JOB_TIME $GPU_COMMAND$EXTRA"
  #--tunnel-hosts turing.wpi.edu \
